unit SizeBox;

{$MESSAGE 'NCAg̈̂ӂTImage TSizeBox'}
interface

uses
  Forms,Windows, Messages, SysUtils, Classes, Controls, ExtCtrls;

type
  TSizePart                   = (rpTitleBar,rpSizeNW,rpSizeN,rpSizeNE,
    rpSizeW,rpSizeE,rpSizeSW ,rpSizeS,rpSizeSE,rpSizeBox,rpSysMenu,
    rpMinimize,rpMaximize,rpClose,rpVScroll,rpHScroll,rpBeep);
  TSizeBox                    = class(TImage)
  private
    { Private 錾 }
    mPart                     : TSizePart;
    mOnMouseLeave             : TNotifyEvent;
    mOnMouseEnter             : TNotifyEvent;

    procedure   SetPart       (value:TSizePart);
  protected
    { Protected 錾 }
    function    GetPartName(): Integer;
    procedure   MouseDown     (Button: TMouseButton;
      Shift: TShiftState; X: Integer; Y: Integer); override;
    procedure   MouseUp(Button: TMouseButton; Shift: TShiftState; X: Integer;
      Y: Integer); override;
    procedure   DblClick; override;

    procedure   CMMouseLeave  (var Message:TMessage); message CM_MOUSELEAVE;
    procedure   CMMouseEnter  (var Message:TMessage); message CM_MOUSEENTER;
  public
    { Public 錾 }
    constructor Create(AOwner: TComponent); override;
  published
    { Published 錾 }
    property    Part          : TSizePart     read  mPart           write SetPart      default rpTitleBar;
    property    OnMouseLeave  : TNotifyEvent  read  mOnMouseLeave   write mOnMouseLeave;
    property    OnMouseEnter  : TNotifyEvent  read  mOnMouseEnter   write mOnMouseEnter;
  end;


procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('݂', [TSizeBox]);
end;

////////////////////////////////////////////////////////////////////////////////
// TSizeBox
////////////////////////////////////////////////////////////////////////////////

constructor TSizeBox.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  mPart := rpTitleBar;
end;

//////

procedure TSizeBox.MouseDown(Button: TMouseButton;
      Shift: TShiftState; X: Integer; Y: Integer);
var BtnName: Integer;
begin
  inherited;
  case Button of
    mbLeft    :BtnName:= WM_NCLBUTTONDOWN;
    mbMiddle  :BtnName:= WM_NCMBUTTONDOWN;
    mbRight   :BtnName:= WM_NCRBUTTONDOWN;
  end;
  ReleaseCapture;
  TControl(Owner).Perform(BtnName,GetPartName,MakeLong(X,Y));
end;

procedure TSizeBox.MouseUp(Button: TMouseButton; Shift: TShiftState; X: Integer;
      Y: Integer);
var BtnName: Integer;
begin
  inherited;
  case Button of
    mbLeft    :BtnName:= WM_NCLBUTTONUP;
    mbMiddle  :BtnName:= WM_NCMBUTTONUP;
    mbRight   :BtnName:= WM_NCRBUTTONDOWN;
  end;
  ReleaseCapture;
  TControl(Owner).Perform(BtnName,GetPartName,MakeLong(X,Y));
end;

procedure TSizeBox.DblClick();
begin
  inherited;
  ReleaseCapture;
  with Mouse.CursorPos do
    TControl(Owner).Perform(WM_NCLBUTTONDBLCLK,GetPartName,MakeLong(X,Y));
end;

procedure TSizeBox.CMMouseLeave(var Message:TMessage);
begin
  inherited;
  if Assigned(OnMouseLeave) then OnMouseLeave(Self);
end;

procedure TSizeBox.CMMouseEnter(var Message:TMessage);
begin
  inherited;
  if Assigned(OnMouseEnter) then OnMouseEnter(Self);
end;

////

procedure TSizeBox.SetPart(value:TSizePart);
begin
  mPart := value;
  case value of
    rpSizeNW  :Cursor:= crSizeNWSE;
    rpSizeN   :Cursor:= crSizeNS;
    rpSizeNE  :Cursor:= crSizeNESW;
    rpSizeW   :Cursor:= crSizeWE;
    rpSizeE   :Cursor:= crSizeWE;
    rpSizeSW  :Cursor:= crSizeNESW;
    rpSizeS   :Cursor:= crSizeNS;
    rpSizeSE  :Cursor:= crSizeNWSE;
    rpSizeBox :Cursor:= crSizeAll;
  else
    Cursor:= crArrow;
  end;
end;

////

function TSizeBox.GetPartName(): Integer;
begin
  case Part of
    rpTitleBar:Result:= HTCAPTION;
    rpSizeNW  :Result:= HTTOPLEFT;
    rpSizeN   :Result:= HTTOP;
    rpSizeNE  :Result:= HTTOPRIGHT;
    rpSizeW   :Result:= HTLEFT;
    rpSizeE   :Result:= HTRIGHT;
    rpSizeSW  :Result:= HTBOTTOMLEFT;
    rpSizeS   :Result:= HTBOTTOM;
    rpSizeSE  :Result:= HTBOTTOMRIGHT;
    rpSizeBox :Result:= HTSIZE;
    rpSysMenu :Result:= HTSYSMENU;
    rpMinimize:Result:= HTMINBUTTON;
    rpMaximize:Result:= HTMAXBUTTON;
    rpClose   :Result:= HTCLOSE;
    rpVScroll :Result:= HTVSCROLL;
    rpHScroll :Result:= HTHSCROLL;
    rpBeep    :Result:= HTERROR;
  end;
end;


end.
