unit stdDialogs;

interface
uses
  Windows, Controls, StdCtrls, Dialogs, Forms,ShellApi,MMSystem;
type
  TDialogProc = procedure(var Dialog:TForm) of object;
  procedure Alert               (Prompt: String;DlgType: TMsgDlgType=mtInformation; HelpCtx: Longint=0);
  function  Confirm             (Prompt: String;DlgType: TMsgDlgType=mtConfirmation; HelpCtx: Longint=0):Boolean;
  function  ConfirmDefNo        (Prompt: String;DlgType: TMsgDlgType=mtConfirmation; HelpCtx: Longint=0):Boolean;
  function  Prompt              (Prompt: String;var Value: String): Boolean;
  procedure TopMost             (Handle: THandle);
  procedure UnTopMost           (Handle: THandle);
  procedure UpdateHandle        (Handle: THandle);
  procedure MsgBeep             (DlgType: TMsgDlgType);
  procedure ShellAbout          (OtherStuff: String='';OpeningSound: Boolean=False);
  function  ShowDialog          (Dialog: TFormClass;PreopenProc: TDialogProc=nil;
      PositiveResultProc: TDialogProc=nil;DefaultTag: Integer=0): TModalResult;
implementation
/////////////////////////
// @ꂫ@
/////////////////////////
// 1.0...Alert,Confirm,MsgBeep֐ǉ
// 1.1...Prompt֐ǉ
// 1.2...ShellAbout֐ǉ
// 1.3...Confirm֐@\ǉ
// 1.4...Confirm֐̋@\𕪊āAConfirmDefNo֐ǉ
// 1.5...TopMost,UnTopMost֐ǉ
// 1.6...UpdateHandle֐ǉ
// 1.7...ShowDialog֐ǉ
// 1.8...ShowDialog֐ɁAȗ\ǉ


procedure Alert(Prompt:String;DlgType: TMsgDlgType=mtInformation; HelpCtx: Longint=0);
begin
  MsgBeep(DlgType);
  MessageDlg(Prompt,DlgType,[mbOk],HelpCtx);
end;

function Confirm(Prompt:String;DlgType: TMsgDlgType=mtConfirmation; HelpCtx: Longint=0):Boolean;
Begin
  MsgBeep(DlgType);
  Result := MessageDlg(Prompt,DlgType,[mbYes,mbNo],HelpCtx) = mrYes
end;

function ConfirmDefNo(Prompt:String;DlgType: TMsgDlgType=mtConfirmation; HelpCtx: Longint=0):Boolean;
Begin
  MsgBeep(DlgType);
  with CreateMessageDialog(Prompt,DlgType,[mbYes, mbNo]) do begin
    TButton(FindComponent('Yes')).Default := false;
    TButton(FindComponent('No')).Default := true;
    Result := ShowModal = mrYes;
    Release;
  end;
end;

function Prompt(Prompt:String;var Value: String): Boolean;
begin
  Result := InputQuery(Application.Title,Prompt,Value);
end;

procedure TopMost(Handle:THandle);
begin
  SetWindowPos(Handle,HWND_TOPMOST,0,0,0,0,SWP_NOSIZE or SWP_NOMOVE or SWP_SHOWWINDOW);
end;

procedure UnTopMost(Handle:THandle);
begin
  SetWindowPos(Handle,HWND_NOTOPMOST,0,0,0,0,SWP_NOSIZE or SWP_NOMOVE or SWP_SHOWWINDOW);
end;

procedure UpdateHandle(Handle:THandle);
begin
  SetWindowPos(Handle,0,0,0,0,0,
    SWP_DRAWFRAME or SWP_NOZORDER or SWP_NOMOVE or SWP_NOSIZE);
end;

procedure MsgBeep(DlgType:TMsgDlgType);
begin
  case DlgType of
    mtWarning     :MessageBeep(MB_ICONWARNING);
    mtError       :MessageBeep(MB_ICONERROR);
    mtInformation :MessageBeep(MB_ICONINFORMATION);
    mtConfirmation:MessageBeep(MB_ICONQUESTION);
  end;
end;

procedure ShellAbout(OtherStuff:String='';OpeningSound:Boolean=false);
begin
  if OpeningSound then
    PlaySound(PChar('SystemStart'),0,SND_ASYNC or SND_ALIAS);
  with Application do
    ShellApi.ShellAbout(MainForm.Handle,PChar(Title),PChar(OtherStuff),Icon.Handle);
end;

function ShowDialog(Dialog:TFormClass;
  PreopenProc,PositiveResultProc: TDialogProc;DefaultTag: Integer): TModalResult;
  //------------------------------------------------------------------------------
  // ʂɃ_CAO\֐
  // Dialogc_CAONX(T̂̂̂܂ܓn)
  // PreopenProcc_CAO\OɌĂяoCxg
  // PositiveResultProcc_CAO"͂"A"OK"ȂǂŕꂽƂɌĂяoCxg
  // DefaultTagc_CAOTag̒l̏l
  // -- TDialogProc
  //   Dialogc\_CAO
  //------------------------------------------------------------------------------
var Dlg : TForm;
begin
  Dlg := Dialog.Create(Application);
  try
    Dlg.Tag := DefaultTag;
    if Assigned(PreopenProc) then PreopenProc(Dlg);
    Result := Dlg.ShowModal;
    if IsPositiveResult(Result) and Assigned(PositiveResultProc) then
      PositiveResultProc(Dlg);
  finally
    Dlg.Free;
  end;
end;

end.
